<?php

namespace EuKit\Base\Parts;

use EuKit\Base\Interfaces\ViewModel;
use Infomaker\Everyware\NewsML\Objects\ImageItem;
use Infomaker\Everyware\Support\NewRelicLog;
use Infomaker\Imengine\CropData\AutoCropData;
use Infomaker\Imengine\CropData\CropResizeData;
use Infomaker\Imengine\Helpers\ImengineHelper;
use Infomaker\Imengine\Imengine;
use RuntimeException;

class Image implements ViewModel
{

    /**
     * @var ImageItem
     */
    private $image;

    /**
     * @var string
     */
    private $ratio = '16:9';

    public function __construct(ImageItem $image)
    {
        $this->image = $image;
    }

    /**
     * @param string $ratio Examples: '16:9', '3:2'
     */
    public function setRatio(string $ratio): void
    {
        $this->ratio = $ratio;
    }

    public function getRatioUrl($width): string
    {
        try {

            if ($this->image->hasCropForRatio($this->ratio)) {
                $crop = $this->image->getCropForRatio($this->ratio);

                $crop_handler = new CropResizeData($crop['width'], $crop['height']);

                $crop_handler
                    ->setCropX($crop['x'])
                    ->setCropY($crop['y'])
                    ->setWidth($width)
                    ->setHeight(ImengineHelper::calculateRatioHeight($width, $this->ratio));

                return Imengine::cropResize($crop_handler)->fromUuid($this->image->uuid);
            }

            if ($this->image->hasOriginalSizes()) {
                $crop_handler = new AutoCropData($this->image->width, $this->image->height, $width);

                $crop_handler->setRatio($this->ratio);

                return Imengine::autoCrop($crop_handler)->fromUuid($this->image->uuid);
            }

            return Imengine::ratio($width, $this->ratio)->fromUuid($this->image->uuid);

        } catch (RuntimeException $e) {
            NewRelicLog::error('Failed to fetch imengine url', $e);
        }

        return '';
    }

    public function getFitURL($width): string {
        try{
            return Imengine::fit($width)->fromUuid($this->image->uuid);
        } catch (RuntimeException $e){
            NewRelicLog::error('Failed to fetch imengine url', $e);
        }

        return '';
    }

    private function createSrcset(array $sizes, bool $crop = true): string
    {
        $srcset = [];
        foreach ($sizes as $size) {
            if (is_int($size)) {
                if ($crop){
                    $srcset[] = $this->getRatioUrl($size) . " {$size}w";
                }else{
                    $srcset[] = $this->getFitURL($size) . " {$size}w";
                }

            }
        }

        return implode(',', $srcset);
    }

    public function getViewData(): array
    {
        return array_replace($this->image->toArray(), [
            'authors' => $this->image->get('authors'),
            'src' => $this->getRatioUrl(1200),
            'fitSrc' => $this->getFitURL(1200),
            'srcset' => $this->createSrcset([300, 600, 900, 1200, 1500]),
            'fitSrcset' => $this->createSrcset([300, 600, 900, 1200, 1500], false)
        ]);
    }
}
